#!/usr/bin/env python3
"""
TCP Flood Detector
Parameters:
    - IP: Interface IP to monitor
    - port: Port to monitor
"""
from scapy.all import *
import time
import collections
import argparse
from datetime import datetime

class TCPFloodDetector:
    def __init__(self, ip, port, threshold=100, window=5):
        self.ip = ip
        self.port = port
        self.threshold = threshold
        self.window = window
        self.connections = collections.defaultdict(int)
        self.start_time = time.time()
        
    def packet_callback(self, packet):
        if TCP in packet and IP in packet:
            if packet[TCP].dport == self.port and packet[IP].dst == self.ip:
                src_ip = packet[IP].src
                current_time = time.time()
                
                # Clean old entries
                if current_time - self.start_time >= self.window:
                    self.connections.clear()
                    self.start_time = current_time
                
                # Count connections
                self.connections[src_ip] += 1
                
                # Check for potential flood
                if self.connections[src_ip] > self.threshold:
                    print(f"\n[{datetime.now()}] WARNING: Potential TCP flood detected!")
                    print(f"Source IP: {src_ip}")
                    print(f"Connection attempts: {self.connections[src_ip]}")
                    print(f"Time window: {self.window} seconds")
                    print("-" * 50)
    
    def start_monitoring(self):
        try:
            print(f"Starting TCP flood detection on {self.ip}:{self.port}")
            print(f"Threshold: {self.threshold} connections per {self.window} seconds")
            print("Press Ctrl+C to stop...\n")
            
            # Start packet capture
            sniff(
                filter=f"tcp and host {self.ip} and port {self.port}",
                prn=self.packet_callback
            )
        except KeyboardInterrupt:
            print("\nMonitoring stopped")
        except Exception as e:
            print(f"\nError: {str(e)}")

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="TCP Flood Detector")
    parser.add_argument("ip", help="Interface IP to monitor")
    parser.add_argument("port", type=int, help="Port to monitor")
    parser.add_argument("--threshold", type=int, default=100,
                      help="Connection threshold (default: 100)")
    parser.add_argument("--window", type=int, default=5,
                      help="Time window in seconds (default: 5)")
    args = parser.parse_args()
    
    detector = TCPFloodDetector(args.ip, args.port, args.threshold, args.window)
    detector.start_monitoring()
